# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)
import logging
from datetime import date

from odoo import fields, models
from odoo.exceptions import ValidationError

_logger = logging.getLogger(__name__)


class AccOperation(models.Model):
    _name = "acc.operation"
    _inherit = ["acc.operation", "api.enedis.acc"]

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def get_perimeter(self):
        self.ensure_one()
        message = self.perimeter()

        # Logs information logs
        log_id = self.env["acc.logs"].create(
            {
                "name": "Appel API Enedis Périmètre "
                + self.name
                + " du "
                + str(fields.Date.today()),
                "date_launched": fields.Datetime.now(),
                "type_log": "api",
                "message": message,
                "acc_operation_id": self.id,
            }
        )

        view_id = self.env.ref("oacc.acc_logs_form").id
        return {
            "name": "LOGS",
            "view_type": "form",
            "view_mode": "form",
            "views": [(view_id, "form")],
            "res_model": "acc.logs",
            "view_id": view_id,
            "type": "ir.actions.act_window",
            "res_id": log_id.id,
            "target": "new",
            "flags": {"initial_mode": "view"},
        }

    # ------------------------------------------------------
    # API functions
    # ------------------------------------------------------
    def perimeter(self):
        """
        Récupère les données de l'opération concernant le périmètre:
         - liste des PRM
         - date de début opération
        """
        # TODO : refactor, too complex
        self._check_access_api()
        message = ""
        message += (
            "<h1>Appel Enedis Périmètre "
            + self.name
            + " du "
            + str(fields.Datetime.now())
            + "</h1>"
        )
        message += "<p><strong>Appel API ...<br/>"
        perimeter_data = self._get_perimeter()
        message += "<p><strong>Appel API terminé<br/>" "Traitement des données ...<br/>"
        usage_points = perimeter_data.get("usage_points")

        list_injection = []
        list_soutirage = []

        for usage_point in sorted(
            usage_points, key=lambda p: date.fromisoformat(p["start"])
        ):
            usage_point_start = date.fromisoformat(usage_point["start"])
            usage_point_end = (
                date.fromisoformat(usage_point["end"])
                if usage_point["end"] != "9999-12-31"
                else False
            )
            usage_point_prm_type = False
            if usage_point["type"] == "CONS":
                usage_point_prm_type = "delivery"
            elif usage_point["type"] == "PROD":
                usage_point_prm_type = "injection"
            message += (
                "<br/>PRM "
                + usage_point["type"]
                + " : "
                + usage_point["usage_point_id"]
                + " - Dates Enedis : "
                + usage_point["start"]
                + " - "
                + usage_point["end"]
                + "<br/>"
            )
            counter_id = self.env["acc.counter"].search(
                [
                    ("name", "=", usage_point["usage_point_id"]),
                ]
            )
            if counter_id and len(counter_id) == 1:
                message += "PRM existe dans Odoo<br/>"
                counter_period_ids = counter_id.period_ids.filtered(
                    lambda p: p.prm_type == usage_point_prm_type
                )
                if counter_period_ids.filtered(
                    lambda p: p.start_date == usage_point_start
                    and p.end_date == usage_point_end
                ):
                    message += (
                        "période existante avec les mêmes dates, pas de modif<br/>"
                    )
                elif counter_period_ids.filtered(
                    lambda p: p.start_date == usage_point_start
                ):
                    counter_period_ids.filtered(
                        lambda p: p.start_date == usage_point_start
                    ).end_date = usage_point_end
                    message += (
                        "période existante avec la même date de début, mais date de "
                        "fin différente, mise à jour date de fin<br/>"
                    )
                else:
                    try:
                        self.env["acc.counter.period"].create(
                            {
                                "acc_counter_id": counter_id.id,
                                "prm_type": usage_point_prm_type,
                                "acc_operation_id": self.id,
                                "start_date": usage_point_start,
                                "end_date": usage_point_end,
                            }
                        )
                        message += (
                            "période inexistante avec les mêmes dates : création<br/>"
                        )
                    except ValidationError as e:
                        message += (
                            "<strong>erreur lors de la tentative de création d'une "
                            "nouvelle période, à vérifier manuellement :</strong><br/>"
                            + str(e)
                            + "<br/>"
                        )

            elif len(counter_id) > 1:
                message += "Plusieurs PRMs trouvés avec ce numéro - pas de modif<br/>"
            else:
                message += "PRM n'existe pas : Création ...<br/>"
                if usage_point_prm_type == "injection":
                    # Si la date de l'opération n'est pas renseignée ou
                    # après la date de démarrage du point d'injection
                    # alors on force la date à celle du point d'injection
                    if (
                        not self.date_start_contract
                        or self.date_start_contract > usage_point_start
                    ):
                        self.date_start_contract = usage_point_start
                try:
                    counter_id = self.env["acc.counter"].create(
                        {
                            "name": usage_point["usage_point_id"],
                        }
                    )
                    self.env["acc.counter.period"].create(
                        {
                            "acc_counter_id": counter_id.id,
                            "prm_type": usage_point_prm_type,
                            "acc_operation_id": self.id,
                            "start_date": usage_point_start,
                            "end_date": usage_point_end,
                        }
                    )
                    message += "Fin de la création du PRM<br/>"
                except ValidationError as e:
                    message += (
                        "<strong>erreur lors de la tentative de création du PRM et/ou "
                        "de la nouvelle période, à vérifier manuellement :<strong><br/>"
                        + str(e)
                        + "<br/>"
                    )
            if usage_point_prm_type == "injection":
                if usage_point["usage_point_id"] not in list_injection:
                    list_injection.append(usage_point["usage_point_id"])
            if usage_point_prm_type == "delivery":
                if usage_point["usage_point_id"] not in list_soutirage:
                    list_soutirage.append(usage_point["usage_point_id"])

        message += "<p>LISTE TOTAL DE PRMs: </br>PRM Injection</br>"

        i = 1
        for inj in list_injection:
            message += str(i) + " - " + inj + "<br/>"
            i += 1
        message += "Total: " + str(len(list_injection)) + "</br>"

        message += "<br/>PRM Soutirage<br/>"
        i = 1
        for inj in list_soutirage:
            message += str(i) + " - " + inj + "<br/>"
            i += 1
        message += "Total: " + str(len(list_soutirage)) + "</br>"

        message += (
            "<h1>Fin appel API Périmètre: " + str(fields.Datetime.now()) + "</h1>"
        )
        return message

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
